package com.kuxuan.laraver.delegates.web.client;

import android.graphics.Bitmap;
import android.webkit.CookieManager;
import android.webkit.WebView;
import android.webkit.WebViewClient;

import com.kuxuan.laraver.app.ConfigKeys;
import com.kuxuan.laraver.app.Laraver;
import com.kuxuan.laraver.delegates.IPageLoadListener;
import com.kuxuan.laraver.delegates.web.WebDelegate;
import com.kuxuan.laraver.delegates.web.route.Router;
import com.kuxuan.laraver.ui.LaraverLoader;
import com.kuxuan.laraver.util.log.LaraverLogger;
import com.kuxuan.laraver.util.storage.LaraverPreference;

/**
 *
 * @author huangfuruixin
 * @date 2017/12/12
 */

public class WebViewClientImpl extends WebViewClient {

    private final WebDelegate DELEGATE;
    private IPageLoadListener mIPageLoadListener = null;

    public void setPageLoadListener(IPageLoadListener listener) {
        this.mIPageLoadListener = listener;
    }

    public WebViewClientImpl(WebDelegate delegate) {
        this.DELEGATE = delegate;
    }

    @Override
    public boolean shouldOverrideUrlLoading(WebView view, String url) {
        LaraverLogger.d("shouldOverrideUrlLoading", url);
        return Router.getInstance().handleWebUrl(DELEGATE, url);
    }

    @Override
    public void onPageStarted(WebView view, String url, Bitmap favicon) {
        super.onPageStarted(view, url, favicon);
        if (mIPageLoadListener != null) {
            mIPageLoadListener.onLoadStart();
        }
        LaraverLoader.showLoading(view.getContext());
    }

    @Override
    public void onPageFinished(WebView view, String url) {
        super.onPageFinished(view, url);
        syncCookie();
        if (mIPageLoadListener != null) {
            mIPageLoadListener.onLoadEnd();
        }

       LaraverLoader.stopLoading();

    }

    private void syncCookie(){
        final CookieManager manager = CookieManager.getInstance();
        /**
         * 注意，这里的Cookie和API请求的Cookie是不一样的，这个在网页不可见
         */
        final String webHost = Laraver.getConfiguration(ConfigKeys.WEB_HOST);
        if(webHost!=null){
            final String cookieStr = manager.getCookie(webHost);
            if(cookieStr!=null&&!cookieStr.equals("")){
                LaraverPreference.addCustomAppProfile("cookie",cookieStr);
            }
        }

    }
}

